import c4d
import socket
import os
import json
from c4d import gui

PLUGIN_ID=1041691

class Request:
    def __init__(self, method, document=None):
        self.id = 2
        self.build = 10
        self.method = method
        self.document = document

class Response:
    def __init__(self, code, message, documents):
        self.code = code
        self.message = message
        self.documents = documents

def send(request, socketPath):
    if not os.path.exists(socketPath): 
        return None
    try:
        client = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        client.connect(socketPath)
        client.sendall(json.dumps(vars(request)).encode("utf-8"))
        client.settimeout(2.0)
        response = client.recv(4096).decode("utf-8")
        client.close()
        dict = json.loads(response)
        return Response(dict["code"], dict["message"], dict["documents"])
    except:
        return None

def exportObjFile(selectionOnly, useMaterial, cachePath):
    objFilePath = os.path.join(cachePath, "sync.obj")
    doc = c4d.documents.GetActiveDocument()
    if selectionOnly:
        objs = doc.GetActiveObjects(c4d.GETACTIVEOBJECTFLAGS_CHILDREN)
        if objs is None: return False
        docTemp = c4d.documents.IsolateObjects(doc, objs)
        if docTemp is None: return  False
        success = c4d.documents.SaveDocument(docTemp,objFilePath,c4d.SAVEDOCUMENTFLAGS_DONTADDTORECENTLIST, 1030178) #c4d.FORMAT_OBJ2EXPORT
        c4d.documents.KillDocument(docTemp)
        if not success: return False
    else:
        success = c4d.documents.SaveDocument(doc,objFilePath,c4d.SAVEDOCUMENTFLAGS_DONTADDTORECENTLIST, 1030178) #c4d.FORMAT_OBJ2EXPORT
        if not success: return False

    if not useMaterial:
        mtlFilePath = os.path.join(cachePath, "sync.mtl")
        if os.path.exists(mtlFilePath):
            os.remove(mtlFilePath)

    return True

def reportError(code, message="Sync Failed"):
    gui.MessageDialog(message + "\nError Code: " + str(code))

class SyncDialogo(c4d.gui.GeDialog):
    unfolderDocuments = []
    socketPath = None
    cachePath = None

    def CreateLayout(self):
        self.GroupBegin(100, c4d.BFH_SCALEFIT, 2, 1)
        self.GroupBorderSpace(8, 8, 8, 0)
        self.AddStaticText(id=5, flags=c4d.BFH_LEFT, name="Sync Document")
        self.AddComboBox(id=4, flags=c4d.BFH_SCALEFIT, initw=80, specialalign=False)
        subid = 0
        for d in self.unfolderDocuments:
            self.AddChild(id=4, subid=subid, child=d)
            subid += 1
        self.GroupEnd()

        self.GroupBegin(100, c4d.BFH_SCALEFIT, 1, 2)
        self.GroupBorderSpace(8, 0, 8, 8)
        self.AddCheckbox(id=2, flags=c4d.BFH_LEFT, initw=200, inith=0, name="Selection Only")
        self.AddCheckbox(id=3, flags=c4d.BFH_LEFT, initw=200, inith=0, name="Use Material")
        self.GroupEnd()

        self.GroupBegin(100, c4d.BFH_RIGHT, 2, 1)
        self.GroupBorderSpace(8, 8, 8, 8)
        self.AddButton(id=1,flags=c4d.BFH_LEFT, initw=0, name="Cancel")
        self.AddButton(id=0,flags=c4d.BFH_LEFT, initw=0, name="Sync")
        self.GroupEnd()
        return True

    def Command(self, id, msg):
        if id == 0:
            syncDocumentId = self.GetInt32(4)
            selectionOnly = self.GetBool(2)
            useMaterial = self.GetBool(3)
            if syncDocumentId is None or syncDocumentId >= len(self.unfolderDocuments) or selectionOnly is None or useMaterial is None:
                self.close
                return False
            syncDocument = self.unfolderDocuments[syncDocumentId]

            success = exportObjFile(selectionOnly, useMaterial, self.cachePath)
            if success:
                res = send(Request(1, syncDocument), self.socketPath)
                if res is None:
                    reportError(202)
                elif res.code != 1:
                    reportError(res.code, res.message)
            else:
                reportError(301)
            self.Close()
            return True
        elif id == 1:
            self.Close()
            return True
        else:
            return True

class UnfolderSyncData(c4d.plugins.CommandData):
    dialog = None

    def Execute(self, doc):
        code = 100
        cachePath  = os.path.expanduser("~/Library/Containers/app.unfolder.Unfolder/Data/Library/Caches/")
        socketPath = os.path.join(cachePath, "sync.socket")
        if os.path.exists(cachePath):
            code += 1
        if not os.path.exists(socketPath):
            cachePath  = os.path.expanduser("~/Library/Caches/app.unfolder.UnfolderL/")
            socketPath = os.path.join(cachePath, "sync.socket")
            if os.path.exists(cachePath):
                code += 2
            if not os.path.exists(socketPath):
                reportError(code, "Cannot Connect to Unfolder\nPlease Ensure Unfolder is Running.")
                return False

        res = send(Request(0), socketPath)
        if res is None:
            reportError(201)
            return False

        if res.code != 0:
            reportError(res.code, res.message)
            return False

        if len(res.documents) == 0:
            reportError(220)
            return False
        
        if self.dialog is None:
            self.dialog = SyncDialogo()
        self.dialog.unfolderDocuments = res.documents
        self.dialog.socketPath = socketPath
        self.dialog.cachePath = cachePath
        self.dialog.Open(dlgtype=c4d.DLG_TYPE_MODAL, pluginid=PLUGIN_ID, defaultw=280, defaulth=10, xpos=-1, ypos=-1)
        return True

if __name__ == "__main__":
    c4d.plugins.RegisterCommandPlugin(id=PLUGIN_ID, str="Unfolder Sync",
                                  help="Sync objects with Unfolder", info=0,
                                  dat=UnfolderSyncData(), icon=None)
